%rainbow_model
% Descartes model of a rainbow. Internal reflection of light rays inside
% spherical raindrops. The primary bow is a single bounce, the secondary
% bow is a double bounce etc.
%
% AF & 5P May 2020.

function rainbow_model
fsize = 18; %Fontsize for plots

%% REFRACTIVE INDEX VS FREQUENCY %%

%Plot the refractive index n of water vs frequency, and overlay the colours.
%Set visible light to be range 405 to 790THz
f = linspace( 405, 790, 200 );
n = nwater(f*1e12);
figure('name','refractive idex vs frequency','color',[ 1,1,1] )
axes('nextplot','add'); set(gca,'fontsize',fsize); 
plot(f,n); set(gca,'fontsize',fsize); xlabel('Frequency /THz'); ylabel('Refractive index');
title('Refractive index of water vs frequency'); grid on; box on;
for p=1:length(f)
    [R,G,B,colour_str] = colours_from_f( f(p)*1e12 );
    plot( f(p), n(p), 'o', 'markerfacecolor',[R,G,B], 'markeredgecolor',[R,G,B] );
end
axis tight; print( gcf, 'n vs frequency.png', '-dpng','-r300');

%

%% RAINBOW ELEVATION VS ANGLE OF INCIDENCE FOR VARIOUS COLOURS OF LIGHT %%

%Generate elevation angle e vs angle of incidence t of light to the normal
%of a raindrop. Plot different curves depending on frequency.
figure('name','elevation angle vs angle of incidence','color',[ 1,1,1] )
axes('nextplot','add'); set(gca,'fontsize',fsize); 
for p=1:length(f)
    
    %Determine colour of lines.
    [R,G,B,colour_str] = colours_from_f( f(p)*1e12 );
    
    %% PRIMARY BOW %%
    
    %Determine epsilon vs angle of incidence
    [e,t,emax_primary(p),tmax] = primary_bow( f(p) );
    
    %Plot e vs t
    plot( t, e, 'color',[R,G,B]);
    
    %% SECONDARY BOW %%
    
    %Determine epsilon vs angle of incidence
    [e,t,emax_secondary(p),tmax] = secondary_bow( f(p) );
    
    %Plot e vs t
    plot( t, e, 'color',[R,G,B]);
    
end
xlabel('Angle of incidence \theta /deg'); ylabel('Angle of elevation \epsilon /deg');
grid on; box on; title('Elevation angle vs angle of incidence'); axis tight; 
print( gcf, 'elevation vs angle of incidence.png', '-dpng','-r300');

%

%% RAINBOW ELEVATION ANGLES VS LIGHT FREQUENCY

figure('name','rainbow elevation angle vs frequency','color',[ 1,1,1] )
axes('nextplot','add'); set(gca,'fontsize',fsize); 
for p=1:length(f)
    [R,G,B,colour_str] = colours_from_f( f(p)*1e12 );
    plot( f(p), emax_primary(p), 'o', 'markerfacecolor',[R,G,B], 'markeredgecolor',[R,G,B] );
    plot( f(p), emax_secondary(p), 'o', 'markerfacecolor',[R,G,B], 'markeredgecolor',[R,G,B] );
end
xlabel('Frequency /THz'); ylabel('Elevation angle /deg'); grid on;
title('Elevation angle of primary (lower) and secondary (upper) rainbows');
axis tight; box on;
print( gcf, 'rainbow elevation vs frequency.png', '-dpng','-r300');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Elevation angle (e) of primary rainbow vs theta (t) given frequency f in THz (10^12 Hz)
function [e,t,emax,tmax] = primary_bow(f)
t = linspace(0,pi/2,1000);  %Define angles of incidence
n = nwater(f*1e12);  %Refractive index
phi = asin( sin(t)/ n );  %Angle of refraction
e = 4*phi - 2*t;  %Elevation
tmax = asin( sqrt( (4 - n^2)/3 ) );  %Angle t at max of e
emax = 4*asin( sin(tmax)/ n ) - 2*tmax;

%Outputs in degrees
t = t*180/pi; tmax = tmax*180/pi; e = e*180/pi; emax = emax*180/pi;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Elevation angle (e) of secondary rainbow vs theta (t) given frequency f in THz (10^12 Hz)
function [e,t,emax,tmax] = secondary_bow(f)
t = linspace(0,pi/2,1000);  %Define angles of incidence
n = nwater(f*1e12);  %Refractive index
phi = asin( sin(t)/ n );  %Angle of refraction
e = pi - 6*phi + 2*t;  %Elevation
tmax = asin( sqrt( (9 - n^2)/8 ) );  %Angle t at max of e
emax = pi - 6*asin( sin(tmax)/ n ) + 2*tmax;

%Outputs in degrees
t = t*180/pi; tmax = tmax*180/pi; e = e*180/pi; emax = emax*180/pi;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Semi-empirical formula for the refractive index of water vs frequency /Hz
function n = nwater(f)
n = sqrt( 1 + ( 1.731 -0.261*(f/1e15).^2 ).^-0.5 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%colours_from_f
% Function which provides the R,G,B values( within interval [0,1] )
% of visible light depending on the frequency /Hz
function [R,G,B,colour_str] = colours_from_f(f)

%Frequency f in THz
f = f/1e12;
if f < 405
    R = NaN; G = NaN; B = NaN; colour_str = 'Infra Red';
elseif (f>=405) && ( f < 480 )
    R = 1; G = 0; B = 0; colour_str = 'Red';
elseif (f>=480) && ( f < 510 )
    R = 1; G = 127/255; B = 0; colour_str = 'Orange';
elseif (f>=510) && ( f < 530 )
    R = 1; G = 1; B = 0; colour_str = 'Yellow';
elseif (f>=530) && ( f < 600 )
    R = 0; G = 1; B = 0; colour_str = 'Green';
elseif (f>=600) && ( f < 620 )
    R = 0; G = 1; B = 1; colour_str = 'Cyan';
elseif (f>=620) && ( f < 680 )
    R = 0; G = 0; B = 1; colour_str = 'Blue';
elseif (f>=680) && ( f <= 790 )
    R = 137/255; G = 0; B = 1; colour_str = 'Violet';
else
    R = NaN; G = NaN; B = NaN; colour_str = 'Ultra Violet';
end

%End of code